from __future__ import annotations

import enum
from datetime import datetime, timezone

from pydantic import BaseModel, Field, field_validator


class ArchiveType(str, enum.Enum):
    LIBCARE_CRASH = 'libcare-crash'
    EPORTAL_DOCTOR = 'eportal-doctor'
    EPORTAL_CHECKINS = 'eportal-checkins'
    UNKNOWN = 'unknown'

    def __str__(self) -> str:
        return self.value


MANIFEST_SCHEMA_VERSION: int = 1


class Manifest(BaseModel, validate_assignment=True):
    schema_version: int = MANIFEST_SCHEMA_VERSION
    archive_type: ArchiveType = Field(alias='type')
    time_created: datetime
    events: list[dict] | None = None
    instance_id: str | None = None

    @field_validator('time_created')
    def to_utc(cls, v: datetime) -> datetime:
        return v.astimezone(timezone.utc)
