from enum import Enum

from flask_login.mixins import AnonymousUserMixin, UserMixin  # type: ignore[import-untyped]

from eportal import perms


class UserAuthType(Enum):
    NONE = 1
    SESSION = 2
    BASIC_AUTH = 3
    API_KEY = 4


class EportalAnonumousUser(AnonymousUserMixin):  # type: ignore[misc]
    auth_type: UserAuthType = UserAuthType.NONE

    @property
    def permissions(self) -> perms.PermSet:
        return set()  # pragma: no cover


class EportalUser(UserMixin):  # type: ignore[misc]
    auth_type: UserAuthType = UserAuthType.SESSION

    @property
    def permissions(self) -> perms.PermSet:
        return set()  # pragma: no cover

    @property
    def is_authenticated(self) -> bool:
        return self.auth_type not in (UserAuthType.NONE, UserAuthType.API_KEY)
