from __future__ import annotations

from werkzeug.datastructures.structures import ImmutableMultiDict
from wtforms.fields import StringField, FieldList, FormField, BooleanField, IntegerField
from wtforms.form import Form
from wtforms.validators import Regexp
from wtforms.widgets import HiddenInput
from typing import Any, Iterable


class ServerTagsForm(Form):
    class TagForm(Form):
        tag_id = IntegerField(widget=HiddenInput())
        assigned = BooleanField()
        tag_value = StringField(
            validators=[Regexp(r'^\w*$', message="Only alphanumeric characters and underscore are valid here")],
        )

    tags = FieldList(FormField(TagForm))

    def __init__(self, formdata: ImmutableMultiDict | None = None, data: Iterable[dict[str, Any]] = ()) -> None:
        super(ServerTagsForm, self).__init__(formdata=formdata)
        for tag in data:
            self.tags.append_entry(data=tag)
