"""Added api_keys table

Revision ID: e3538a4f68e1
Revises: 88d8933cf2ab
Create Date: 2024-12-05 12:46:42.751190

"""

# revision identifiers, used by Alembic.
revision = 'e3538a4f68e1'
down_revision = '88d8933cf2ab'
branch_labels = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table(
        'api_keys',
        sa.Column('id', sa.Integer(), nullable=False, primary_key=True, autoincrement=True),
        sa.Column('name', sa.String(), nullable=False),
        sa.Column('key_hash', sa.String(), nullable=False),
        sa.Column('created', sa.DateTime(), nullable=False, server_default=sa.func.current_timestamp()),
        sa.Column('kc_user_id', sa.Integer, nullable=True),
        sa.Column('unit_user_id', sa.Integer, nullable=True),
        sa.Column('expiration', sa.DateTime(), nullable=True),
        sa.CheckConstraint(
            '(kc_user_id IS NULL) != (unit_user_id IS NULL)',
            name='ck_api_keys_users_discriminator',
        ),
        sa.UniqueConstraint('name', 'kc_user_id', name='uq_api_keys_name_kc_user_id'),
        sa.UniqueConstraint('name', 'unit_user_id', name='uq_api_keys_name_unit_user_id'),
        sa.ForeignKeyConstraint(['kc_user_id'], ['kc_users.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['unit_user_id'], ['unit_users.id'], ondelete='CASCADE'),
    )


def downgrade():
    op.drop_table('api_keys')
