from __future__ import annotations

from typing import Any, Generic, TypeVar

from wtforms.form import Form
from flask_admin.base import BaseView
from flask_admin.model.base import ViewArgs, OrderedDict
from jinja2.runtime import Context
from sqlalchemy.orm import Query, Session, scoped_session
from .filters import BaseSQLAFilter

Model = TypeVar('Model')

class ModelView(BaseView, Generic[Model]):
    def __init__(self, view: type[Model], session: scoped_session[Session], name: str): ...
    def _export_data(self) -> tuple[int, list[Model]]: ...
    def get_list_value(self, context: Context, model: Model, name: str) -> Any: ...
    def get_detail_value(self, context: Context, model: Model, name: str) -> Any: ...
    def get_export_value(self, model: Model, name: str) -> Any: ...
    def get_one(self, id: str) -> Model | None: ...
    def get_query(self) -> Query: ...
    def get_count_query(self) -> Query: ...
    def handle_view_exception(self, exc: Exception) -> bool: ...
    def edit_form(self, obj: Model | None = None) -> Form: ...
    def create_form(self) -> Form: ...
    def on_model_change(self, form: Form, model: Model, is_created: bool) -> None: ...
    def delete_model(self, model: Model) -> bool: ...
    def handle_filter(self, filter: BaseSQLAFilter) -> BaseSQLAFilter: ...
    def init_search(self) -> bool: ...
    def _get_list_extra_args(self) -> ViewArgs: ...
    def _apply_search(
        self,
        query: Query,
        count_query: Query | None,
        joins: dict[Any, Any],
        count_joins: dict[Any, Any] | None,
        search: str,
    ) -> tuple[Query, Query | None, dict[Any, Any], dict[Any, Any] | None]: ...
    def _apply_filters(
        self,
        query: Query,
        count_query: Query | None,
        joins: dict[Any, Any],
        count_joins: dict[Any, Any] | None,
        filters: list[Any] | None,
    ) -> tuple[Query, Query | None, dict[Any, Any], dict[Any, Any] | None]: ...
    def get_url(self, endpoint: str) -> str: ...
    def validate_form(self, form: Form) -> bool: ...
    def _get_filter_groups(self) -> OrderedDict: ...

    _filter_joins: dict[Any, list[Any]]
    _filters: list[Any]
    _auto_joins: list[Any]
    _search_fields: list[tuple[Any, list[Any]]]
    _search_supported: bool
